@@ -90,6 +90,10 @@ def save(filename: str, variable: Any) -> None:
     Tuples are also supported but will be loaded back as lists, without
     warning.
 
+    See also
+    --------
+    ktk.load
+
     """
 
     class CustomEncoder(json.JSONEncoder):
@@ -260,6 +264,11 @@ def load(filename: str, *, include_metadata: bool = False) -> Any:
     -------
     Any
         The loaded variable.
+
+    See also
+    --------
+    ktk.save
+
     """
     archive = zipfile.ZipFile(filename, "r")
 
@@ -315,6 +324,10 @@ def read_c3d(
     This function, which has been introduced in version 0.9, is still
     experimental and its behaviour or API may change slightly in the future.
 
+    See also
+    --------
+    ktk.write_c3d
+
     Notes
     -----
     - This function relies on `ezc3d`, which is available on
@@ -551,8 +564,12 @@ def write_c3d(filename: str, points: TimeSeries, **kwargs) -> None:
         Each point position is expressed as an Nx4 point series. Events from
         this TimeSeries are also added to the c3d.
 
-    Notes
-    -----
+    See also
+    --------
+    ktk.read_c3d
+
+    Note
+    ----
     This function relies on `ezc3d`, which is available on
     conda-forge and on git-hub. Please install ezc3d before using
     write_c3d. https://github.com/pyomeca/ezc3d
