@@ -21,7 +21,7 @@ Provide 3d geometry and linear algebra functions related to biomechanics.
 Note
 ----
 As a convention, the first dimension of every array is always N and corresponds
-to time. For constants, use a length of 1 as the first dimension.
+to time.
 
 """
 
@@ -31,10 +31,22 @@ __email__ = "chenier.felix@uqam.ca"
 __license__ = "Apache 2.0"
 
 
+def __dir__():
+    return [
+        "matmul",
+        "create_transforms",
+        "get_angles",
+        "create_frames",
+        "get_local_coordinates",
+        "get_global_coordinates",
+        "isnan",
+        "register_points",
+    ]
+
+
 import numpy as np
 import scipy.spatial.transform as transform
 import kineticstoolkit.external.icp as icp
-from kineticstoolkit.decorators import unstable, directory
 from typing import Optional, Tuple
 
 
@@ -43,11 +55,8 @@ def matmul(op1: np.ndarray, op2: np.ndarray) -> np.ndarray:
     Matrix multiplication between series of matrices.
 
     This function is a wrapper for numpy's matmul function (operator @), that
-    helps numpy to understand Kinetics Toolkit's convention that the first
-    dimension always corresponds to time.
-
-    It aligns and create additionnal dimensions if needed to avoid dimension
-    mismatch errors.
+    uses Kinetics Toolkit's convention that the first dimension always
+    corresponds to time, to broadcast time correctly between operands.
 
     Parameters
     ----------
@@ -59,9 +68,23 @@ def matmul(op1: np.ndarray, op2: np.ndarray) -> np.ndarray:
     Returns
     -------
     np.ndarray
-        The product, as a series of Nx4 or Nx4xM matrices.
+        The product, usually as a series of Nx4 or Nx4xM matrices.
+
+    Example
+    -------
+    A matrix multiplication between one matrix and a series of 3 vectors
+    results in a series of 3 vectors.
+
+    >>> import kineticstoolkit as ktk
+    >>> mat_series = np.array([[[2.0, 0.0], [0.0, 1.0]]])
+    >>> vec_series = np.array([[4.0, 5.0], [6.0, 7.0], [8.0, 9.0]])
+    >>> ktk.geometry.matmul(mat_series, vec_series)
+    array([[ 8.,  5.],
+           [12.,  7.],
+           [16.,  9.]])
 
     """
+
     def perform_mul(op1, op2):
         if isinstance(op1, np.ndarray) and isinstance(op2, np.ndarray):
             return op1 @ op2
@@ -83,18 +106,14 @@ def matmul(op1: np.ndarray, op2: np.ndarray) -> np.ndarray:
     return result
 
 
-def create_transforms(seq: Optional[str] = None,
-                      angles: Optional[np.ndarray] = None,
-                      translations: Optional[np.ndarray] = None,
-                      *,
-                      degrees=False) -> np.ndarray:
+def create_transforms(
+    seq: Optional[str] = None,
+    angles: Optional[np.ndarray] = None,
+    translations: Optional[np.ndarray] = None,
+    degrees=False,
+) -> np.ndarray:
     """
-    Create an Nx4x4 series of homogeneous transforms.
-
-    Warning
-    -------
-    This function, which has been introduced in 0.4, is still experimental and
-    may change signature or behaviour in the future.
+    Create an Nx4x4 series of homogeneous transforms based on angle sequences.
 
     Parameters
     ----------
@@ -106,26 +125,23 @@ def create_transforms(seq: Optional[str] = None,
         Required if angles is specified.
 
     angles
-        Optional array_like, shape (N,) or (N, [1 or 2 or 3]).
-        Angles are specified in radians (degrees is False) or degrees (degrees
-        is True).
+        Optional array_like of shape (N,) or (N, [1 or 2 or 3]). Angles are
+        specified in radians (if degrees is False) or degrees (if degrees is
+        True).
 
-        For a single character seq, angles can be:
+        For a single-character `seq`, `angles` can be:
 
-            - array_like with shape (N,), where each angle[i] corresponds to a
-              single rotation
-            - array_like with shape (N, 1), where each angle[i, 0] corresponds
-              to a single rotation
+        - array_like with shape (N,), where each `angle[i]` corresponds to a
+          single rotation;
+        - array_like with shape (N, 1), where each `angle[i, 0]` corresponds
+          to a single rotation.
 
-        For 2- and 3-character wide seq, angles can be:
-
-            - array_like with shape (W,) where W is the width of seq, which
-              corresponds to a single rotation with W axes
-            - array_like with shape (N, W) where each angle[i] corresponds to
-              a sequence of Euler angles describing a single rotation
+        For 2- and 3-character `seq`, `angles` is an array_like with shape
+        (N, W) where each `angle[i, :]` corresponds to a sequence of Euler
+        angles and W is the length of `seq`.
 
     translations
-        Optional float or array_like, shape (N, 3) or (N, 4). This corresponds
+        Optional array_like or shape (N, 3) or (N, 4). This corresponds
         to the translation part of the generated series of homogeneous
         transforms.
 
@@ -137,6 +153,10 @@ def create_transforms(seq: Optional[str] = None,
     np.ndarray
         An Nx4x4 series of homogeneous transforms.
 
+    See also
+    --------
+    ktk.geometry.create_frames
+
     Example
     -------
     Create a series of two homogeneous transforms that rotates 0, then 90
@@ -164,7 +184,7 @@ def create_transforms(seq: Optional[str] = None,
     # Condition angles
     if angles is None:
         angles = np.array([0])
-        seq = 'x'
+        seq = "x"
     else:
         angles = np.array(angles)
 
@@ -187,26 +207,15 @@ def create_transforms(seq: Optional[str] = None,
     return T
 
 
-def get_angles(T: np.ndarray,
-               seq: str,
-               degrees: bool = False,
-               flip: bool = False) -> np.ndarray:
+def get_angles(
+    T: np.ndarray, seq: str, degrees: bool = False, flip: bool = False
+) -> np.ndarray:
     """
-    Represent a series of transformation matrices as series of Euler angles.
+    Extract Euler angles from a series of homogeneous matrices.
 
-    This function is a wrapper for scipy.transform.Rotation.as_euler. Please
-    consult scipy help for the complete docstring.
-
-    Euler angles suffer from the problem of gimbal lock, where the
-    representation loses a degree of freedom and it is not possible to
-    determine the first and third angles uniquely. In this case, a warning is
-    raised, and the third angle is set to zero. Note however that the returned
-    angles still represent the correct rotation.
-
-    Warning
-    -------
-    This function, which has been introduced in 0.4, is still experimental and
-    may change signature or behaviour in the future.
+    In case of gimbal lock, a warning is raised, and the third angle is set to
+    zero. Note however that the returned angles still represent the correct
+    rotation.
 
     Parameters
     ----------
@@ -220,38 +229,45 @@ def get_angles(T: np.ndarray,
         intrinsic rotations cannot be mixed in one function call.
 
     degrees
-        Returned angles are in degrees if this flag is True, else they are in
+        If True, the returned angles are in degrees. If False, they are in
         radians. Default is False.
 
     flip
-        Return an alternate sequence with the second angle inverted, that
-        leads to the same rotation matrices. More specifically:
-
-        First angle belongs to [-180, 180] degrees (both inclusive)
-
-        Second angle belongs to:
-            - Default case (alt_angles = False):
-                - [-90, 90] degrees if all axes are different (like xyz)
-                - [0, 180] degrees if first and third axes are the same
-                  (like zxz)
-            - Alternate case (alt_angles = True):
-                - [-180, -90], [90, 180] degrees if all axes are different
-                  (like xyz)
-                - [-180, 0] degrees if first and third axes are the same
-                  (like zxz)
-
-        Third angle belongs to [-180, 180] degrees (both inclusive)
-
-        One rationale for adding this special case is the calculation
-        of shoulder angles: when following the ISB recommendation (YXY), X
-        corresponds to the negative elevation and thus we expect to get
-        negative values for Y. In this case, just use alt_angles = True.
+        Return an alternate sequence with the second angle inverted, but that
+        leads to the same rotation matrices. See below for more information.
 
     Returns
     -------
     np.ndarray
-        A Tx3 array of Euler angles.
+        An Nx3 series of Euler angles, with the second dimension containing
+        the first, second and third angles, respectively.
 
+    Notes
+    -----
+    The range of the returned angles is dependant on the `flip` parameter. If
+    `flip` is False:
+
+    - First angle belongs to [-180, 180] degrees (both inclusive)
+    - Second angle belongs to:
+
+        - [-90, 90] degrees if all axes are different. e.g., xyz
+        - [0, 180] degrees if first and third axes are the same e.g., zxz
+
+    - Third angle belongs to [-180, 180] degrees (both inclusive)
+
+    If `flip` is True:
+
+    - First angle belongs to [-180, 180] degrees (both inclusive)
+    - Second angle belongs to:
+
+        - [-180, -90], [90, 180] degrees if all axes are different. e.g., xyz
+        - [-180, 0] degrees if first and third axes are the same e.g., zxz
+
+    - Third angle belongs to [-180, 180] degrees (both inclusive)
+
+    This function is a wrapper for scipy.transform.Rotation.as_euler. Please
+    consult scipy help for more help on intrinsic/extrinsic angles and the
+    `seq` parameter.
 
     """
     R = transform.Rotation.from_matrix(T[:, 0:3, 0:3])
@@ -267,102 +283,60 @@ def get_angles(T: np.ndarray,
         else:  # Tait-Bryan angles
             angles[:, 0] = np.mod(angles[:, 0], 2 * offset) - offset
             angles[:, 1] = offset - angles[:, 1]
-            angles[angles[:, 1] > offset, :] -= (2 * offset)
+            angles[angles[:, 1] > offset, :] -= 2 * offset
             angles[:, 2] = np.mod(angles[:, 2], 2 * offset) - offset
 
     return angles
 
 
 def create_frames(
-        origin: np.ndarray,
-        x: Optional[np.ndarray] = None,
-        y: Optional[np.ndarray] = None,
-        z: Optional[np.ndarray] = None,
-        xy: Optional[np.ndarray] = None,
-        xz: Optional[np.ndarray] = None,
-        yz: Optional[np.ndarray] = None) -> np.ndarray:
+    origin: np.ndarray,
+    x: Optional[np.ndarray] = None,
+    y: Optional[np.ndarray] = None,
+    z: Optional[np.ndarray] = None,
+    xy: Optional[np.ndarray] = None,
+    xz: Optional[np.ndarray] = None,
+    yz: Optional[np.ndarray] = None,
+) -> np.ndarray:
     """
-    Create a Nx4x4 series of frames based on series of points and vectors.
-
-    Create reference frames based on points and vectors and return this series
-    of reference frames as a series of transformation matrices.
-
-    This function's behaviour is better explained using an example. We will
-    create reference frames for the right humerus, based on the recommendations
-    of the International Society of Biomechanics [1]. Lets say we have
-    constructed series of points for the glenohumeral joint (GH), lateral elbow
-    epicondyle (EL) and medial elbow epicondyle (EM). Following the ISB:
-
-        1. The origin is GH;
-        2. The y axis is the line between GH and the midpoint of EL and EM,
-           pointing to GH;
-        3. The x axis is the normal to the GH-EL-EM plane, pointing forward;
-           which means that GH-EL-EM is a yz plane.
-        4. The z axis is perpendicular to x and y, pointing to the right.
-
-    Therefore:
-
-        1. origin = GH
-        2. y = GH - (EL + EM) / 2
-        3. yz = EL - EM  # The x axis is formed by cross(y, yz)
-        4. reference_frames = ktk.geometry.create_reference_frames(
-            origin=origin, y=y, yz=yz)
-
-    Warning
-    -------
-    This function, which has been introduced in 0.4, is still experimental and
-    may change signature or behaviour in the future.
+    Create an Nx4x4 series of frames based on series of points and vectors.
 
     Parameters
     ----------
     origin
         A series of N points (Nx4) that corresponds to the origin of the
-        returned reference frames.
+        series of frames to be created.
 
     x|y|z
-        A series of N vectors (Nx4) that are aligned toward the {x|y|z}
-        axis of the returned reference frames.
+        Define either `x`, `y` or `z`. A series of N vectors (Nx4) that
+        are aligned toward the {x|y|z} series of frames to be created.
 
     xy|xz
-        When x is specified, a series of N vectors (Nx4) in the {xy|xz} plane
-        of the returned reference frames.
+        Only if `x` is specified. A series of N vectors (Nx4) in the {xy|xz}
+        plane of the series of frames to be created. As a rule of thumb, use
+        a series of N vectors that correspond roughly to the {z|-y} axis.
 
     xy|yz
-        When y is specified, a series of N vectors (Nx4) in the {xy|yz} plane
-        of the returned reference frames.
+        Only if `y` is specified. A series of N vectors (Nx4) in the {xy|yz}
+        plane of the series of frames to be created. As a rule of thumb, use
+        a series of N vectors that correspond roughly to the {z|x} axis.
 
     xz|yz
-        When z is specified, a series of N vectors (Nx4) in the {xz|yz} plane
-        of the returned reference frames.
+        Only if `z` is specified. A series of N vectors (Nx4) in the {xz|yz}
+        plane of the series of frames to be created. As a rule of thumb, use
+        a series of N vectors that correspond roughly to the {-y|x} axis.
 
     Returns
     -------
     np.ndarray
-        Series of transformation matrices (Nx4x4).
+        Series of frames (Nx4x4).
 
-    Examples
+    See also
     --------
-    Create a translated reference frame using 3 points:
-
-        >>> import kineticstoolkit.lab as ktk
-        >>> origin = [[2., 2., 2., 1.]]
-        >>> x = [[10., 0., 0., 0.]]
-        >>> xy = [[10., 10., 0., 0.]]
-        >>> rf = ktk.geometry.create_frames(origin, x=x, xy=xy)
-        >>> rf
-        array([[[1., 0., 0., 2.],
-                [0., 1., 0., 2.],
-                [0., 0., 1., 2.],
-                [0., 0., 0., 1.]]])
-
-    References
-    ----------
-    1. G. Wu et al., "ISB recommendation on definitions of joint
-       coordinate systems of various joints for the reporting of human joint
-       motion - Part II: shoulder, elbow, wrist and hand," Journal of
-       Biomechanics, vol. 38, no. 5, pp. 981--992, 2005.
+    ktk.geometry.create_transforms
 
     """
+
     def normalize(v):
         """Normalize series of vectors."""
         norm = np.linalg.norm(v, axis=1)
@@ -415,8 +389,9 @@ def create_frames(
     return np.stack((v_x, v_y, v_z, origin), axis=2)
 
 
-def get_local_coordinates(global_coordinates: np.ndarray,
-                          reference_frames: np.ndarray) -> np.ndarray:
+def get_local_coordinates(
+    global_coordinates: np.ndarray, reference_frames: np.ndarray
+) -> np.ndarray:
     """
     Express global coordinates in local reference frames.
 
@@ -440,6 +415,10 @@ def get_local_coordinates(global_coordinates: np.ndarray,
         Series of local coordinates in the same shape than
         `global_coordinates`.
 
+    See also
+    --------
+    ktk.geometry.get_global_coordinates
+
     """
     n_samples = global_coordinates.shape[0]
 
@@ -473,8 +452,9 @@ def get_local_coordinates(global_coordinates: np.ndarray,
     return local_coordinates
 
 
-def get_global_coordinates(local_coordinates: np.ndarray,
-                           reference_frames: np.ndarray) -> np.ndarray:
+def get_global_coordinates(
+    local_coordinates: np.ndarray, reference_frames: np.ndarray
+) -> np.ndarray:
     """
     Express local coordinates in the global reference frame.
 
@@ -498,13 +478,16 @@ def get_global_coordinates(local_coordinates: np.ndarray,
         Series of global coordinates in the same shape than
         `local_coordinates`.
 
+    See also
+    --------
+    ktk.geometry.get_local_coordinates
+
     """
     global_coordinates = np.zeros(local_coordinates.shape)
     global_coordinates = matmul(reference_frames, local_coordinates)
     return global_coordinates
 
 
-@unstable
 def isnan(input: np.ndarray, /) -> np.ndarray:
     """
     Check which samples has at least one NaN.
@@ -523,12 +506,13 @@ def isnan(input: np.ndarray, /) -> np.ndarray:
     """
     temp = np.isnan(input)
     while len(temp.shape) > 1:
-        temp = (temp.sum(axis=1) > 0)
+        temp = temp.sum(axis=1) > 0
     return temp
 
 
-def _match_size(op1: np.ndarray, op2: np.ndarray
-                ) -> Tuple[np.ndarray, np.ndarray]:
+def _match_size(
+    op1: np.ndarray, op2: np.ndarray
+) -> Tuple[np.ndarray, np.ndarray]:
     """
     Match the first dimension of op1 and op2.
 
@@ -539,8 +523,8 @@ def _match_size(op1: np.ndarray, op2: np.ndarray
 
     Returns
     -------
-    2x np.ndarray
-        References or copies of op1 and op2 now matched in size.
+    Tuple of two np.ndarray
+        op1 and op2, now matched in size.
 
     """
     if op1.shape[0] == 1:
@@ -550,22 +534,17 @@ def _match_size(op1: np.ndarray, op2: np.ndarray
         op2 = np.repeat(op2, op1.shape[0], axis=0)
 
     if op1.shape[0] != op2.shape[0]:
-        raise ValueError(
-            'Could not match first dimension of op1 and op2')
+        raise ValueError("Could not match first dimension of op1 and op2")
 
     return op1, op2
 
 
-def register_points(global_points: np.ndarray,
-                    local_points: np.ndarray) -> np.ndarray:
+def register_points(
+    global_points: np.ndarray, local_points: np.ndarray
+) -> np.ndarray:
     """
     Find the homogeneous transforms between two series of point clouds.
 
-    Warning
-    -------
-    This function, which has been introduced in 0.4, is still experimental and
-    may change signature or behaviour in the future.
-
     Parameters
     ----------
     global_points : array of shape Nx4xM
@@ -580,6 +559,12 @@ def register_points(global_points: np.ndarray,
         Array of shape Nx4x4, expressing a series of 4x4 homogeneous
         transforms.
 
+    Warning
+    -------
+    This function, which has been introduced in 0.4, is still experimental and
+    may change signature or behaviour in the future. Namely, the names of the
+    parameters may change in the future.
+
     """
     n_samples = global_points.shape[0]
 
@@ -592,35 +577,33 @@ def register_points(global_points: np.ndarray,
         # Identify which global points are visible
         sample_global_points = global_points[i_sample]
         sample_global_points_missing = np.isnan(
-            np.sum(sample_global_points, axis=0))
+            np.sum(sample_global_points, axis=0)
+        )
 
         # Identify which local points are visible
         sample_local_points = local_points[i_sample]
         sample_local_points_missing = np.isnan(
-            np.sum(sample_local_points, axis=0))
+            np.sum(sample_local_points, axis=0)
+        )
 
         sample_points_missing = np.logical_or(
-            sample_global_points_missing, sample_local_points_missing)
+            sample_global_points_missing, sample_local_points_missing
+        )
 
         # If at least 3 common points are visible between local and global
         # points, then we can regress the transformation.
         if sum(~sample_points_missing) >= 3:
             T[i_sample] = icp.best_fit_transform(
                 sample_local_points[0:3, ~sample_points_missing].T,
-                sample_global_points[0:3, ~sample_points_missing].T)[0]
+                sample_global_points[0:3, ~sample_points_missing].T,
+            )[0]
         else:
             T[i_sample] = np.nan
 
     return T
 
 
-module_locals = locals()
-
-
-def __dir__():  # pragma: no cover
-    return directory(module_locals)
-
-
 if __name__ == "__main__":  # pragma: no cover
     import doctest
+
     doctest.testmod(optionflags=doctest.NORMALIZE_WHITESPACE)
